% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{default_msal_token_cache}
\alias{default_msal_token_cache}
\title{Get default MSAL token cache path}
\usage{
default_msal_token_cache()
}
\value{
A character string with the path to the MSAL token cache file.
}
\description{
Returns the path to the MSAL token cache file shared by the Azure CLI and
Azure SDKs. Defaults to \code{msal_token_cache.json} inside the Azure config
directory (see \code{\link[=default_azure_config_dir]{default_azure_config_dir()}}).
}
\seealso{
\code{\link[=default_azure_config_dir]{default_azure_config_dir()}}, \code{\link[=write_msal_token]{write_msal_token()}}
}
