% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/credential-azure-cli.R
\name{az_cli_logout}
\alias{az_cli_logout}
\title{Azure CLI Logout}
\usage{
az_cli_logout()
}
\value{
Invisibly returns \code{NULL}
}
\description{
Logs out from Azure CLI by removing all stored credentials and account
information. This function runs \verb{az logout}.
}
\details{
After logging out, you will need to run \code{\link[=az_cli_login]{az_cli_login()}} again to
authenticate and use Azure CLI credentials.
}
