% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{default_azure_client_id}
\alias{default_azure_client_id}
\title{Get default Azure client ID}
\usage{
default_azure_client_id()
}
\value{
A character string with the client ID
}
\description{
Retrieves the Azure client ID in priority order:
\enumerate{
\item \code{AZURE_CLIENT_ID} environment variable
\item Built-in fallback (Microsoft's public Azure CLI client ID)
}
}
\examples{
default_azure_client_id()
}
