% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-client.R
\name{default_non_auth}
\alias{default_non_auth}
\title{Default No Authentication}
\usage{
default_non_auth(req)
}
\arguments{
\item{req}{An \code{\link[httr2:request]{httr2::request()}} object}
}
\value{
The same \code{\link[httr2:request]{httr2::request()}} object, unmodified
}
\description{
A pass-through credential function that performs no authentication.
This function returns the request object unchanged, allowing API calls
to be made without adding any authentication headers or tokens.
}
