% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{default_azure_config_dir}
\alias{default_azure_config_dir}
\title{Get default Azure configuration directory}
\usage{
default_azure_config_dir()
}
\value{
A character string with the Azure configuration directory path
}
\description{
Retrieves the Azure configuration directory from the \code{AZURE_CONFIG_DIR}
environment variable, or falls back to the platform-specific default.
}
\examples{
default_azure_config_dir()
}
