% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{default_azure_host}
\alias{default_azure_host}
\title{Get default Azure authority host}
\usage{
default_azure_host()
}
\value{
A character string with the authority host URL
}
\description{
Retrieves the Azure authority host in priority order:
\enumerate{
\item \code{AZURE_AUTHORITY_HOST} environment variable
\item Built-in fallback (\code{login.microsoftonline.com})
}
}
\examples{
default_azure_host()
}
