% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{default_azure_tenant_id}
\alias{default_azure_tenant_id}
\title{Get default Azure tenant ID}
\usage{
default_azure_tenant_id()
}
\value{
A character string with the tenant ID
}
\description{
Retrieves the Azure tenant ID in priority order:
\enumerate{
\item \code{AZURE_TENANT_ID} environment variable
\item Built-in fallback (\code{"common"})
}
}
\examples{
default_azure_tenant_id()
}
