% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{azure_client}
\alias{azure_client}
\title{Azure Default Client Configuration}
\format{
An object of class \code{list} of length 2.
}
\usage{
azure_client
}
\description{
Default client ID and tenant ID used for Azure authentication when not
explicitly provided. The client ID is Microsoft's public Azure CLI client ID.
}
\keyword{internal}
