% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/credential-azure-cli.R
\name{az_cli_is_login}
\alias{az_cli_is_login}
\title{Check if User is Logged in to Azure CLI}
\usage{
az_cli_is_login(timeout = 10L)
}
\arguments{
\item{timeout}{A numeric value specifying the timeout in seconds for the
Azure CLI command. Defaults to \code{10}.}
}
\value{
A logical value: \code{TRUE} if the user is logged in, \code{FALSE} otherwise
}
\description{
Checks whether the user is currently logged in to Azure CLI by attempting
to retrieve account information.
}
