% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{default_azure_scope}
\alias{default_azure_scope}
\title{Get default Azure OAuth scope}
\usage{
default_azure_scope(resource = "azure_arm")
}
\arguments{
\item{resource}{A character string specifying the Azure resource. Must be one of:
\code{"azure_arm"} (Azure Resource Manager), \code{"azure_graph"} (Microsoft Graph),
\code{"azure_storage"} (Azure Storage), or \code{"azure_key_vault"} (Azure Key Vault).
Defaults to \code{"azure_arm"}.}
}
\value{
A character string with the OAuth scope URL
}
\description{
Returns the default OAuth scope for a specified Azure resource.
}
\examples{
default_azure_scope()
default_azure_scope("azure_graph")
}
